/* (2,k)-cuckoo hashing su interi.
 *  
 * Tavola di hash con inserimento "stile cuculo" su interi positivi.
 * Utilizza d funzioni di hash e p elementi per bucket.
 *
 * Le funzioni di hash sono del tipo 
 * h(x) = ((ax + b) mod p) mod N se opera su N secchi.
 *
 * Alessio Orlandi
 */


#include <stdio.h>
#include <stdlib.h>


/* Elenco di primi per le funzioni di hash. */
int primi[2] = { 699757, 899051 };

/* Parametri di una funzione di hash. */
typedef struct {
   int a; /* Fattore moltiplicativo. */
   int b; /* Fattore additivo. */
   int p; /* Primo per il modulo. */
} funzione;

typedef struct {
  int P; /* Numero di elementi per secchio. */

  /* Numero di secchi. */
  int nsecchi;

  /* Array di nsecchi puntatori ai secchi.
   * tavola[i] e' l'i-esimo secchio ed e' NULL
   * se non vi e' alcun elemento all'interno.
   * se tavola[i] non e' NULL e' un array di P interi.
   * interi, dove le posizioni non occupate
   * sono sempre mantenute a -1. */
  int **tavola;

  /* Array di parametri per 2 funzioni di hash. */
  funzione param[2];

  /* Numero di elementi in totale. */
  int carico;

  /* Numero di collisioni. */
  int ncollisioni; 

} hash;

void inizializza ( hash *h, int P, int nsecchi )
{
  int i;

  h->P = P;
  h->nsecchi = nsecchi;
  h->ncollisioni = 0;
  h->carico = 0;

  /* Inizializziamo la tavola */
  h->tavola = (int **)malloc(sizeof(int *)*nsecchi);

  for(i = 0; i < h->nsecchi; i++ ) 
	h->tavola[i] = NULL;

  /* Inizializziamo le funzioni. */
  for ( i = 0; i < 2; i++ ) 
  {
      /* Ricordiamo che p e' un primo,
       * a e' il fattore moltiplicativo
       * e b e' un fattore additivo,
       * quindi e' necessario che sia piu' piccolo di a. 
       * Attenzione che a non  deve essere 0. */
      h->param[i].p = primi[i];
      h->param[i].a = 1+random()%(h->param[i].p-1);
      h->param[i].b = random()%h->param[i].a;
  }
}

void calcola( hash *h, int elemento, int *risultati)
{
  int i = 0;
  for (i = 0; i < 2; i++ ) 
    risultati[i] = (( h->param[i].a * elemento +
                   h->param[i].b ) % (h->param[i].p)) %
		   h->nsecchi;
}

void scambia ( int *x, int *y )
{
   int tmp = *x;
   *x = *y;
   *y = tmp;
}

int aggiungi ( hash *h, int valore, int secchio)
{
  int i;

  if ( h->tavola[secchio] == NULL ) /* Vuoto! */
  {
      h->tavola[secchio] = (int *)malloc(sizeof(int) * h->P);      
      for ( i = 0; i < h->P ; i++ ) 
	 h->tavola[secchio][i] = -1;
  }

  for ( i = 0; i < h->P; i++ ) 
     if ( h->tavola[secchio][i] == -1 ) 
     {
          h->tavola[secchio][i] = valore;
          return 1;
     }

 return 0;
}

/* Inserimento nella tavola di un elemento >= 0. 
 * Restituisce 0 se l'elemento gia' era inserito,
 * -1 se l'inserimento e' fallito, o il numero
 * di iterazioni del ciclo di scambi fatte 
 * ( > 0) nell'altro caso. 
 */
int inserisci ( hash *h, int elemento )
{
  int i,j,k;
  int x, pos; /* Elemento corrente e posizione corrente. */
  int hx[2];  /* Gli hash dell'elemento corrente. */

  /* Prima di tutto controlliamo che non ci sia. */
  if ( cerca (h, elemento) != -1 )
	return 0;
  
  x = elemento; 
  calcola(h, elemento, hx);
  pos = hx[0]; /* x viene sicuramente inserito in questo posto. */

  h->carico++;

  /* Proviamo a inserire fino al numero massimo di secchi. */
  for ( i = 0; i < h->nsecchi; i++ ) 
  {
      if ( aggiungi( h, x, pos ) ) 
	return i+1; /* fatto. */
	
      h->ncollisioni++;

      /* No, dobbiamo scambiare. Decidiamo di farlo col primo. */
      scambia( &x, &h->tavola[pos][0]);
	
      /* Ora cerchiamo la prossima posizione da esaminare. */
      calcola ( h, x, hx ) ;
      if ( pos == hx[0] ) pos = hx[1]; 
      else pos = hx[0];
  }


  return -1;
}

int cerca ( hash *h, int elemento )
{
   int hx[2], i, j;
   calcola ( h, elemento, hx );

   for ( i = 0; i < 2; i++ ) 
     for ( j = 0; h->tavola[ hx[i] ] != NULL && j < h->P; j++)
	if ( h->tavola[ hx[i] ][j] == elemento ) return i;

   return -1;
}


int main()
{
   int N, D, op, x, risultato;
   hash h; 
   

   scanf("%d %d", &N, &D);
   inizializza(&h, N, D);

   while(1) 
   {
      scanf("%d %d", &op, &x);
      if ( op == 0 ) /* Inserimento */
        printf("%d\n", inserisci ( &h, x ), h.carico);
      else 
        printf("%d\n", cerca( &h, x ));
   }

  return 0;
}

